@@ -1,7 +1,7 @@
 import sqlalchemy as sa
 from sqlalchemy_utils.functions import create_database
-from pyg_base import cfg_read, as_list, dictable, Dict, is_dict, is_dictable, is_strs, is_str, is_int, is_date, dt2str, ulist, try_back, unique
-from pyg_encoders import as_reader, as_writer, dumps, loads, encode, decode
+from pyg_base import cache, cfg_read, as_list, dictable, Dict, is_dict, is_dictable, is_strs, is_str, is_int, is_date, dt2str, ulist, try_back, unique
+from pyg_encoders import as_reader, as_writer, dumps, loads
 from sqlalchemy import Table, Column, Integer, String, MetaData, Identity, Float, DATE, DATETIME, TIME, select, func, not_, desc, asc
 from sqlalchemy.orm import Session
 import datetime
@@ -12,17 +12,33 @@ _doc = 'doc'
 _root = 'root'
 _deleted = 'deleted'
 
-DRIVER = None
-SERVER = None
 
+@cache
+def _servers():
+    return cfg_read().get('sql_server', {})
+
+@cache
+def _schema():
+    return cfg_read().get('sql_schema', None)
+
+@cache
+def _database():
+    return cfg_read().get('sql_database', 'master')
+
+    
 def get_server(server = None):
     """
-    determines the sql server striing
+    Determines the sql server string
+    We support a server config which looks like:
+        config['sql_server'] = dict(dev = 'server.test', prod = 'prod.server')
     """
-    if server is None or server is True:
-        server = SERVER
-    if server is None or server is True:
-        server = cfg_read().get('sql_server')
+    servers = _servers()
+    if isinstance(servers, str):
+        if server is None or server is True:
+            server = servers
+    else:
+        server = servers.get(server, server)
+        server = servers.get(server, server) #double redirection supported
     if server is None:
         raise ValueError('please provide server or set a "sql_server" in cfg file: from pyg_base import *; cfg = cfg_read(); cfg["sql_server"] = "server"; cfg_write(cfg)')
     return server
@@ -31,7 +47,6 @@ def get_driver(driver = None):
     """
     determines the sql server driver
     """
-    driver = driver or DRIVER
     if driver is None or driver is True:
         driver = cfg_read().get('sql_driver')
     if driver is None:
@@ -51,23 +66,26 @@ def get_driver(driver = None):
 
 
 def _pairs2connection(*pairs, **connection):
-    for p in pairs:
-        k, v = p.split('=')
-        k = k.strip()
-        v = v.strip()
-        connection[k] = v
+    connection = connection.copy()
+    for pair in pairs:
+        ps = pair.split(';')
+        for p in ps:
+            k, v = p.split('=')
+            k = k.strip()
+            v = v.strip().replace("'","")
+            connection[k] = v
     connection = {k.lower() : v.replace(' ','+').replace('{','').replace('}','') for k, v in connection.items() if v is not None}
     return connection
 
 def _db(connection):
     db = connection.pop('db', None)
     if db is None:
-        db = connection.pop('database', 'master')
+        db = connection.pop('database', _database())
     return db
     
 def get_cstr(*pairs, **connection):
     """
-    determines the connection string, at the moment 'user' and 'password' are not used and are mere place-holders
+    determines the connection string
     """
     connection = _pairs2connection(*pairs, **connection)
     server = get_server(connection.pop('server', None))
@@ -178,7 +196,8 @@ def sql_table(table, db = None, non_null = None, nullable = None, _id = None, sc
         table_name = table 
     else:
         table_name = table.name
-        schema = schema or table.schema
+        schema = table.schema
+    schema = schema or _schema()
     if doc is True:
         doc = _doc
     meta = MetaData()
@@ -212,8 +231,7 @@ def sql_table(table, db = None, non_null = None, nullable = None, _id = None, sc
         tbl = Table(table_name, meta, *cols)
         meta.create_all(e)
     else:
-        meta.reflect(e)
-        tbl = meta.tables[table_name]
+        tbl = Table(table_name, meta, autoload_with = e, schema = schema)
         cols = tbl.columns
         non_nulls = [Column(k, _types.get(t, t), nullable = False) for k, t in pks.items()]
         if non_nulls is not None:
